package operation

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong"
	"shylinux.com/x/enterprise/src/guanlixitong"
	"shylinux.com/x/operation/src/operation/model"
)

type Table struct {
	guanlixitong.Table
	list string `name:"list cloud_uid uid auto" role:"void"`
}

func (s Table) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.USER_CLOUD_ROLE:
		s.InputsListRole(m, UserCloudRoleList, arg...)
	case model.CLOUD_TYPE:
		s.InputsList(m, CloudTypeList, arg...)
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s Table) RewriteAppend(m *ice.Message, arg ...string) *ice.Message {
	m.RewriteAppend(func(value, key string, index int) string {
		switch key {
		case model.USER_CLOUD_ROLE:
			value = UserCloudRole(kit.Int(value)).String()
		case model.CLOUD_TYPE:
			value = CloudType(kit.Int(value)).String()
		case model.PRICE:
			value = kit.Format("%.2f", kit.Float(value)/100)
		}
		return value
	})
	return s.Table.RewriteAppend(m)
}
func (s Table) CheckRole(m *ice.Message, arg ...string) {
	role := UserCloudRole(s.UserPlaceRole(m))
	m.WarnNotRight(!kit.IsIn(role.String(), append(arg, UserCloudCreator.String())...), role.String())
}

type Tables struct{ Table }

func (s Tables) BeforeMigrate(m *ice.Message, arg ...string) {}

func NewPortal(userPlace gonganxitong.UserPlacer, place gonganxitong.Placer) Portal {
	return Portal{Portal: guanlixitong.NewPortal(userPlace, place)}
}
