package operation

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"2025-dev.shylinux.com/x/20250211-service/src/gateway"
	"2025-dev.shylinux.com/x/20250215-cluster/src/travel"
	"shylinux.com/x/operation/src/operation/model"
)

type Gateway struct {
	Table
	cluster       Cluster
	travel        travel.Travel
	gateway       gateway.Gateway
	order         string `data:"2"`
	fields        string `data:"title,content,space,cluster_uid,user_uid"`
	payfor        string `name:"payfor" help:"购买" style:"notice" role:"leader"`
	createProject string `name:"createProject name* repos binary" help:"创建" style:"notice" role:"worker"`
	scanProject   string `name:"scanProject" help:"扫描" role:"worker"`
	open          string `name:"open" role:"leader"`
	modify        string `name:"modify title content" style:"danger" role:"leader"`
}

func (s Gateway) List(m *ice.Message, arg ...string) {
	if s.ValueList(m, arg); s.IsLeader(m) {
		m.PushAction(s.CreateProject, s.ScanProject, s.Open, s.Modify)
	} else if s.IsWorker(m) {
		m.PushAction(s.CreateProject, s.ScanProject)
	} else {
		m.PushAction()
	}
	s.Fields(m, model.UID, s.KeyAS(s.cluster, model.SPACE)).SelectJoin(m, s.cluster, s.Keys(s.cluster, model.SPACE))
	if m.Action(); m.Length() == 0 {
		m.Echo("请到「云集群」购买主机")
	} else {
		m.Display("")
	}
	if len(arg) == 2 && s.IsLeader(m) {
		m.EchoIFrame(web.S(s.space(m.Spawn().Options(model.CLOUD_UID, arg[0], model.UID, arg[1]))))
	}
}
func (s Gateway) CreateProject(m *ice.Message, arg ...string) {
	m.Cmdy(project{}, s.Create, arg, model.GATEWAY_UID, m.Option(model.UID), kit.Dict(web.SPACE, s.space(m))).ProcessRefresh()
}
func (s Gateway) ScanProject(m *ice.Message, arg ...string) {
	m.Cmdy(project{}, s.Scan, kit.Dict(model.GATEWAY_UID, m.Option(model.UID), web.SPACE, s.space(m.Spawn()))).ProcessRefresh()
}
func (s Gateway) Open(m *ice.Message, arg ...string) {
	m.ProcessOpen(m.Options(ice.MSG_USERPOD, "").MergePodCmd(s.space(m), web.ADMIN))
}
func (s Gateway) Modify(m *ice.Message, arg ...string) {
	s.Update(m, arg, m.OptionSimple(model.CLOUD_UID, model.UID)...)
	s.RecordEventWithName(m, "")
}

func init() { ice.TeamCtxCmd(Gateway{}) }

func (s Gateway) Payfor(m *ice.Message, arg ...string) {
	s.Transaction(m, func() {
		msg := m.Spawn()
		if msg = s.Orders(msg, mdb.ID).Limit(msg, 1).SelectForUpdate(msg, "cluster_uid = ? AND (cloud_uid IS NULL OR cloud_uid = '')", m.Option(model.CLUSTER_UID)); msg.Length() == 0 {
			m.Echo("主机不够用啦！请等待管理员添加。")
		} else {
			m.Option(model.TITLE, "我的云主机")
			s.Update(m, m.OptionSimple(model.CLOUD_UID, model.USER_UID, model.TITLE), msg.AppendSimple(model.UID)...)
			s.RecordEventWithName(m.Options(model.UID, msg.Append(model.UID)), "")
			s.SendMessage(s.GetCommandUID(m), "", "")
		}
	})
}
func (s Gateway) Scan(m *ice.Message, arg ...string) {
	m.Cmd(web.SPACE, m.Option(web.SPACE), s.travel, s.travel.Concurrent, ice.GetTypeMod(s.gateway), ice.GetTypeKey(s.gateway)).Table(func(value ice.Maps) {
		s.InsertIfNeed(m, web.SPACE, value[web.SPACE], model.CLUSTER_UID, m.Option(model.CLUSTER_UID))
	})
	s.Fields(m, "count(*) AS total").Select(m, model.CLUSTER_UID, m.Option(model.CLUSTER_UID))
	s.Fields(m, "count(*) AS stock").Select(m, "cluster_uid = ? AND (cloud_uid IS NULL OR cloud_uid = '')", m.Option(model.CLUSTER_UID))
}
func (s Gateway) space(m *ice.Message, arg ...string) string {
	s.Tables(m, s.cluster).Fields(m, s.Key(s, web.SPACE), s.KeyAS(s.cluster, web.SPACE))
	s.Select(m, kit.Simple(m.OptionSimple(model.CLOUD_UID), s.Key(s, model.UID), m.Option(model.UID))...)
	return kit.Keys(m.Append(model.CLUSTER_SPACE), m.Append(web.SPACE))
}
